/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from '../../../../state/reducers'
import {getI18n} from '../../../../../i18n/i18nRoot'
import {Button, Header, Icon, List, Modal} from 'semantic-ui-react'
import {FilePreviewFromBackend} from '../../../../types/exerciseEditor'
import {convertSizeToHumanReadableSize} from '../../../../helpers/convertersAndTransformers'
import {SimpleVDivider} from '../../../helpers/simpleVDivider'
import MaterialSearchInput from '../../../material/materialSearchInput'
import {InputOnChangeEvent} from '../../../../types/reactEvents'
import Spinner from '../../../helpers/spinner'
import {HelpPopup} from '../../../helpers/helpPopup'
import Logger from '../../../../helpers/logger'
import {askDialog, errorDialog} from '../../../../helpers/dialogHelper'
import {MyPopup} from '../../../helpers/myPopup'
import {leftIconPopupOffset} from '../../../../constants'
import orderBy from 'lodash-es/orderBy'

//const css = require('./styles.styl');

export interface MyProps {

  /**
   * the title for the modal
   */
  readonly title: string

  /**
   * an id for the input so that we can get a reference
   */
  readonly uploadInputId: string

  /**
   * true: the dialog is displayed
   * false: not
   */
  readonly isDisplayed: boolean
  /**
   * true: show loader, false: not
   */
  readonly isBusy: boolean

  /**
   * the asset file that are already attached
   */
  readonly attachedAssetFilePreviews: ReadonlyArray<FilePreviewFromBackend>

  /**
   * these file previews are not detachable
   */
  readonly notDetachableAssetFilePreviews?: ReadonlyArray<FilePreviewFromBackend>

  /**
   * the reason that some previews are not detachable
   */
  readonly notDetachableReason?: string | null

  /**
   * all attachable asset files
   */
  readonly allAssetFilePreviews: ReadonlyArray<FilePreviewFromBackend>

  /**
   * called when a file is attached
   */
  readonly onAttachAssetFilePreview: (assetFilePreview: FilePreviewFromBackend) => Promise<void>

  /**
   * called when a file is detached
   */
  readonly onDetachAssetFilePreview: (assetFilePreview: FilePreviewFromBackend) => Promise<void>

  /**
   * the filter text (so filter all asset previews)
   */
  readonly filterText: string

  /**
   * called when the filter text changes
   */
  readonly onFilterTextChanged: (val: string) => void

  /**
   * called when a file is uploaded
   */
  readonly onUploadFile: (file: File) => Promise<void>
  /**
   * null to hide prog bar
   * active will show 100% but with progress 'animation'
   */
  readonly uploadPercentage: number | null | 'active'

  /**
   * cancel is disabled if we have ceil(100%)
   */
  readonly onUploadCanceled: () => void

  /**
   * called when the dialog should be closed
   */
  readonly onClose: () => void

}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    ...props,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here

                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class addAssetFilesModal extends React.Component<Props, any> {

  async onUploadFile(): Promise<void> {

    const input = document.getElementById(this.props.uploadInputId) as HTMLInputElement

    const file: File = input.files[0]

    try {
      await this.props.onUploadFile(file)
    } catch (err) {
      //ignore... is probably handled inside
    }

    input.value = ""

  }

  render(): JSX.Element {
    return (
      <div>

        {/*not really hosted here, this is created in an outer most div */}
        <Modal open={this.props.isDisplayed} size="large" closeOnDimmerClick={false}>

          {
            this.props.isBusy &&
            <Spinner
              percentage={this.props.uploadPercentage}
              onCancel={(this.props.uploadPercentage !== null
                && typeof this.props.uploadPercentage === 'number' && this.props.uploadPercentage < 100)
                        ? this.props.onUploadCanceled
                        : null}
            />
          }

          <Header>
            {
              this.props.title
            }
          </Header>

          {
            //height is taken from ui.modal .scrolling.content
          }
          <Modal.Content scrolling style={{height: 'calc(70vh)'}}>

            <div className="flexed fh">

              {
                //list with asset files that can be added
              }
              <div className="flexed-h fh" style={{
                maxWidth: '450px',
                minWidth: '300px'
              }}>

                <div className="flexed" style={{
                  justifyContent: 'space-between',
                  alignItems: 'baseline'
                }}>

                  <div className="flexed-header-with-tooltip">
                    <h3 className="mar-right-half">{getI18n(this.props.langId,
                                                            "All Files"
                    )} ({this.props.allAssetFilePreviews.length})</h3>
                    <HelpPopup
                      defaultText={getI18n(this.props.langId,
                                           "These are all files where you have access to. For exercises or exercise tests you need permission to change the exercise."
                      )}/>
                  </div>

                  <div className="clickable" onClick={() => {

                    const input = document.getElementById(this.props.uploadInputId)
                    if (!input) {
                      Logger.error(`Could not get html element with specific id '${this.props.uploadInputId}'`)
                      errorDialog(getI18n(this.props.langId, 'Error'),
                                  getI18n(this.props.langId, 'Could not get html element with specific id'),
                                  this.props.langId
                      )
                      return
                    }

                    input.click()

                  }}>
                    <Icon size="large" name="upload"/>
                    <input id={this.props.uploadInputId} type="file" multiple={false} className="collapsed"
                           accept="*/*"
                           onChange={() => {
                             this.onUploadFile()
                           }}
                    />
                  </div>

                </div>

                <div className="mar-bot">
                  <MaterialSearchInput
                    value={this.props.filterText}
                    onChange={(e: InputOnChangeEvent) => {
                      this.props.onFilterTextChanged(e.currentTarget.value)
                    }}
                  />
                </div>

                <div className="scrollable">
                  <List>
                    {
                      //filter out all that are already attached
                      orderBy(
                        this.props.allAssetFilePreviews
                          .filter(
                            p => p.originalName.toLowerCase().indexOf(this.props.filterText.toLowerCase()) !== -1),
                        p => p.originalName.toLowerCase()
                      )
                        .map((assetFilePreview: FilePreviewFromBackend) => {

                          const alreadyAdded = this.props.attachedAssetFilePreviews.some(
                            p => assetFilePreview.id === p.id)

                          return (
                            <List.Item key={assetFilePreview.id} className={alreadyAdded
                                                                            ? 'div-disabled'
                                                                            : ''}>
                              <List.Icon className="clickable" name='add' size='large' verticalAlign='middle'
                                         onClick={() => {

                                           if (alreadyAdded) return

                                           this.props.onAttachAssetFilePreview(assetFilePreview)
                                         }}
                              />
                              <List.Content>
                                <List.Header>
                                  <div className="flexed">
                                    <div className="mar-right-half brake-words">
                                      {
                                        assetFilePreview.originalName
                                      }
                                    </div>

                                    <div className="not-important-colored">
                                      (
                                      {convertSizeToHumanReadableSize(assetFilePreview.sizeInBytes)}
                                      )
                                    </div>
                                  </div>
                                </List.Header>
                              </List.Content>
                            </List.Item>
                          )
                        })
                    }
                  </List>

                </div>

              </div>

              <SimpleVDivider/>

              {
                //current asset files attached to the exercise
              }
              <div style={{flex: '1'}}>

                <div className="flexed-header-with-tooltip">
                  <h3 className="mar-right-half">{getI18n(this.props.langId,
                                                          "Attached Files"
                  )} ({this.props.attachedAssetFilePreviews.length})</h3>
                  <HelpPopup defaultText={getI18n(this.props.langId,
                                                  "Note that when you detach a file and it has no connection to another exercise, it will get deleted!"
                  )}/>
                </div>

                <List>
                  {
                    orderBy(this.props.attachedAssetFilePreviews, p => p.originalName.toLowerCase())
                      .map((assetFilePreview: FilePreviewFromBackend) => {

                        const cannotBeDetached = this.props.notDetachableAssetFilePreviews !== undefined && this.props.notDetachableAssetFilePreviews.some(
                          p => p.id === assetFilePreview.id)

                        return (
                          <List.Item key={assetFilePreview.id}>

                            {
                              cannotBeDetached &&
                              <MyPopup
                                trigger={
                                  <List.Icon className="hoverable" name="question circle" size='large'
                                             verticalAlign='middle'/>
                                }
                                on="hover"
                                horizontalOffset={leftIconPopupOffset}
                                content={this.props.notDetachableReason}
                              />

                            }
                            {
                              !cannotBeDetached &&
                              <List.Icon className="clickable" name='trash' size='large' verticalAlign='middle'
                                         onClick={async () => {

                                           const should = await askDialog(getI18n(this.props.langId, "Detach file"),
                                                                          getI18n(this.props.langId,
                                                                                  "Are you sure you want to detach the file? If this file is not connected to another exercise the fille will get deleted!"
                                                                          ), this.props.langId
                                           )

                                           if (!should) return

                                           this.props.onDetachAssetFilePreview(assetFilePreview)
                                         }}
                              />
                            }


                            <List.Content>
                              <List.Header>
                                <div className="flexed">
                                  <div className="mar-right-half brake-words">
                                    {
                                      assetFilePreview.originalName
                                    }
                                  </div>

                                  <div className="not-important-colored">
                                    (
                                    {convertSizeToHumanReadableSize(assetFilePreview.sizeInBytes)}
                                    )
                                  </div>
                                </div>
                              </List.Header>
                              <List.Description>
                                <div className="flexed">
                                  <div className="mar-right-half not-important-colored">
                                    {
                                      assetFilePreview.mimeType
                                    }
                                  </div>
                                </div>
                              </List.Description>
                            </List.Content>
                          </List.Item>
                        )
                      })
                  }
                </List>

              </div>

            </div>

          </Modal.Content>

          <Modal.Actions>

            <Button
              content={getI18n(this.props.langId, 'Close')}
              onClick={() => {

                if (this.props.isBusy) return

                this.props.onClose()
              }}
            >
            </Button>

          </Modal.Actions>
        </Modal>

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(addAssetFilesModal)
